function Ks = Kstar(D)
% Ks = Kstar(curve) returns the matrix representing the kernel of Kstar com
% puted on the C2boundary D.
% Recall that K^star[f](x) = \frac{1}{2 \pi} \int_{\partial D} \frac{ <x - y, \nu_x> }{|x -
% y |^2 } f(y) d \sigma(y).

% Compute K, the adjoint of Ks
M = D.nbPoints;
Ks = zeros(M, M);

for j = 1:M
    xdoty = (D.points(1,j)-D.points(1,:)).*D.normal(1,j)...
        +(D.points(2,j)-D.points(2,:)).*D.normal(2,j);
    
    norm_xy_square = (D.points(1,j)-D.points(1,:)).^2 ...
        +(D.points(2,j)-D.points(2,:)).^2;
    
    %off-diagonal terms
    Ks(j, 1:j-1) = 1/(2*pi)* ...
        xdoty(1:j-1)./norm_xy_square(1:j-1).*D.sigma(1:j-1);
    Ks(j, j+1:M) = 1/(2*pi)* ...
        xdoty(j+1:M)./norm_xy_square(j+1:M).*D.sigma(j+1:M);
    
    %diagonal term
    Ks(j,j) = -1/(4*pi)* ...
        D.avec(:,j)'*D.normal(:,j)/(D.tvec_norm(j)^2)*D.sigma(j);
end

end